defmodule Fulib.Const do
  #### http://elixir-lang.org/docs/master/elixir/Regex.html
  @ip_regex ~r"^(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9])\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[0-9])$"
  @email_regex ~r"\A([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})\z"
  @pure_number_regex ~r"\A[0-9]*\z"
  @integer_regex ~r"^-?[1-9]\d*$"
  @words_regex ~r/[\w-_0-9]+/u
  @email_regex ~r"\A([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})\z"
  @mobile_number_formats [
    cn: %{
      length: 11,
      regex: ~r/^(1[3-9][0-9])\d{8}$/u
    }
  ]
  @website_url_regex ~r"^((http|https)\:\/\/|[a-zA-Z0-9\.\-]+\.)[0-9a-zA-Z]([-.\w]*[0-9a-zA-Z])*(:(0-9)*)*(\/?)([a-zA-Z0-9\-\=\&\.\?\,\'\/\\\+&amp;%\$#_]*)?"

  @spider_regex ~r(spider|bot|crawler|ysearch\/slurp)

  @top_domains [
    "arpa",
    "asia",
    "biz",
    "cam",
    "cat",
    "com",
    "coop",
    "edu",
    "gov",
    "int",
    "info",
    "jobs",
    "media",
    "mil",
    "mobi",
    "mtn",
    "museum",
    "name",
    "net",
    "pro",
    "travel",
    "wtf",
    "win",
    "xxx",
    "xyz",
    "singles",
    "tel",
    "org",
    "as",
    "bi",
    "ca",
    "co",
    "ed",
    "go",
    "in",
    "jo",
    "me",
    "mi",
    "mo",
    "mt",
    "mu",
    "na",
    "ne",
    "or",
    "pr",
    "te",
    "tr",
    "wt",
    "wi",
    "xx",
    "xy",
    "si",
    "cn",
    "hk",
    "tw",
    "eg",
    "sa",
    "lk",
    "th",
    "ac",
    "ad",
    "af",
    "ag",
    "ai",
    "al",
    "am",
    "an",
    "ao",
    "aq",
    "at",
    "au",
    "aw",
    "az",
    "ba",
    "bb",
    "bd",
    "be",
    "bf",
    "bg",
    "bh",
    "bj",
    "bm",
    "bn",
    "bo",
    "br",
    "bs",
    "bt",
    "bv",
    "bw",
    "by",
    "bz",
    "cc",
    "cd",
    "cf",
    "cg",
    "ch",
    "ci",
    "ck",
    "cl",
    "cm",
    "cr",
    "cs",
    "cu",
    "cv",
    "cx",
    "cy",
    "cz",
    "de",
    "dj",
    "dk",
    "dm",
    "do",
    "dz",
    "ec",
    "ee",
    "eh",
    "er",
    "es",
    "et",
    "fi",
    "fj",
    "fk",
    "fm",
    "fo",
    "fr",
    "ga",
    "gb",
    "gd",
    "ge",
    "gf",
    "gh",
    "gi",
    "gl",
    "gm",
    "gn",
    "gp",
    "gq",
    "gr",
    "gs",
    "gt",
    "gu",
    "gw",
    "gy",
    "hm",
    "hn",
    "hr",
    "ht",
    "hu",
    "id",
    "ie",
    "il",
    "io",
    "iq",
    "ir",
    "is",
    "it",
    "jm",
    "jp",
    "ke",
    "kg",
    "kh",
    "ki",
    "km",
    "kn",
    "kp",
    "kr",
    "kw",
    "ky",
    "kz",
    "la",
    "lb",
    "lc",
    "li",
    "lr",
    "ls",
    "lt",
    "lu",
    "lv",
    "ly",
    "ma",
    "mc",
    "md",
    "mg",
    "mh",
    "mk",
    "ml",
    "mm",
    "mn",
    "mp",
    "mq",
    "mr",
    "ms",
    "mv",
    "mw",
    "mx",
    "my",
    "mz",
    "nc",
    "nf",
    "ng",
    "ni",
    "nl",
    "no",
    "np",
    "nr",
    "nu",
    "nz",
    "om",
    "pa",
    "pe",
    "pf",
    "pg",
    "ph",
    "pk",
    "pl",
    "pm",
    "pn",
    "pt",
    "pw",
    "py",
    "qa",
    "re",
    "ro",
    "ru",
    "rw",
    "sb",
    "sc",
    "sd",
    "se",
    "sg",
    "sh",
    "sj",
    "sk",
    "sl",
    "sm",
    "sn",
    "so",
    "sr",
    "ss",
    "st",
    "sv",
    "sx",
    "sy",
    "sz",
    "tc",
    "td",
    "tf",
    "tg",
    "tj",
    "tk",
    "tm",
    "tn",
    "to",
    "tp",
    "tt",
    "tv",
    "tz",
    "ua",
    "ug",
    "uk",
    "us",
    "uy",
    "uz",
    "va",
    "vc",
    "ve",
    "vg",
    "vi",
    "vn",
    "vu",
    "wf",
    "ws",
    "ye",
    "yt",
    "yu",
    "za",
    "zm",
    "zr",
    "zw",
    "ps",
    "tl",
    "gg",
    "gz",
    "im",
    "um",
    "eu",
    "je",
    "ax",
    "cw",
    "rs",
    "su",
    "rf",
    "bl",
    "bq",
    "mf",
    "bu",
    "dd",
    "ia",
    "ae",
    "ar"
  ]

  @scan_url_regex [
    "((https?|mailto|thunder|ftp|sftp|svn):\/\/[-A-Za-z0-9+&@#\/%\?=~_|!:,.;]+[-A-Za-z0-9+&@#\/%=~_|])|",
    "([a-zA-Z0-9\.\-]+)",
    "(\\.(#{@top_domains |> Enum.join("|")}))",
    "(:(0-9)*)*(\/[a-zA-Z0-9\-\=\&\.\?\,\'\/\\\+&amp;%\$#_]*)?"
  ]

  def spider_regex, do: @spider_regex

  @doc "IO地址的正则表达式"
  def ip_regex, do: @ip_regex

  def ip_match?(ip) do
    Regex.match?(@ip_regex, ip)
  end

  @doc "电子邮箱的正则表达式"
  def email_regex, do: @email_regex

  @doc "纯数字的正则表达式"
  def pure_number_regex, do: @pure_number_regex

  @doc "整数的正则表达式"
  def integer_regex, do: @integer_regex

  @doc "文字正则"
  def words_regex, do: @words_regex

  @doc "只匹配文字，下划线和中划线, 数字"
  def name_regex_match?(str) do
    String.replace(str, @words_regex, "") |> String.length() == 0
  end

  @doc "半角和全角得空格"
  def space_strings, do: [" ", "　"]

  @doc "匹配URL的正则表达式"
  def match_url_regex, do: Regex.compile!("^#{@scan_url_regex |> Enum.join()}")

  @doc """
  扫描出URL的正则表达式
  ```
  eg:
  * https://msdn.microsoft.com/en-us/library/ff650303.aspx#paght000001_commonregularexpressions
  * http://stackoverflow.com/questions/1323283/how-to-match-url-in-c
  ```
  """
  def scan_url_regex, do: Regex.compile!(@scan_url_regex |> Enum.join())

  @doc "web url format"
  def website_url_regex, do: @website_url_regex

  @doc "手机号码的正则表达式，按照国家归属"
  def mobile_number_formats, do: @mobile_number_formats

  def normalize_mobile_number!(region_key, mobile_number) do
    if mobile_number_format = @mobile_number_formats |> Fulib.get(region_key) do
      length_require = mobile_number_format |> Fulib.get(:length)

      mobile_number =
        mobile_number
        |> Fulib.to_s()
        |> String.trim()

      length = String.length(mobile_number)

      cond do
        length > length_require -> String.slice(mobile_number, -length_require, length_require)
        true -> mobile_number
      end
    else
      mobile_number
    end
  end

  def mobile_number_match?(nil, _mobile_number), do: false
  def mobile_number_match?(_region_key, nil), do: false

  def mobile_number_match?(region_key, mobile_number) do
    if mobile_number_format = @mobile_number_formats |> Fulib.get(region_key) do
      Regex.match?(mobile_number_format[:regex], mobile_number)
    else
      false
    end
  end

  @nations [
    cn: %{code: 86, zh_name: "中国", en_name: "China"},
    ad: %{code: 376, zh_name: "安道尔", en_name: "Andorra"},
    ae: %{code: 971, zh_name: "阿拉伯联合酋长国", en_name: "United Arab Emirates"},
    af: %{code: 93, zh_name: "阿富汗", en_name: "Afghanistan"},
    ag: %{code: 1268, zh_name: "安提瓜和巴布达", en_name: "Antigua and Barbuda"},
    ai: %{code: 1264, zh_name: "安圭拉", en_name: "Anguilla"},
    al: %{code: 355, zh_name: "阿尔巴尼亚", en_name: "Albania"},
    am: %{code: 374, zh_name: "亚美尼亚", en_name: "Armenia"},
    ao: %{code: 244, zh_name: "安哥拉", en_name: "Angola"},
    ar: %{code: 54, zh_name: "阿根廷", en_name: "Argentina"},
    as: %{code: 1684, zh_name: "美属萨摩亚", en_name: "American Samoa"},
    at: %{code: 43, zh_name: "奥地利", en_name: "Austria"},
    au: %{code: 61, zh_name: "澳大利亚", en_name: "Australia"},
    aw: %{code: 297, zh_name: "阿鲁巴", en_name: "Aruba"},
    az: %{code: 994, zh_name: "阿塞拜疆", en_name: "Azerbaijan"},
    ba: %{code: 387, zh_name: "波斯尼亚和黑塞哥维那", en_name: "Bosniaand Herzegovina"},
    bb: %{code: 1246, zh_name: "巴巴多斯", en_name: "Barbados"},
    bd: %{code: 880, zh_name: "孟加拉国", en_name: "Bangladesh"},
    be: %{code: 32, zh_name: "比利时", en_name: "Belgium"},
    bf: %{code: 226, zh_name: "布基纳法索", en_name: "Burkina Faso"},
    bg: %{code: 359, zh_name: "保加利亚", en_name: "Bulgaria"},
    bh: %{code: 973, zh_name: "巴林", en_name: "Bahrain"},
    bi: %{code: 257, zh_name: "布隆迪", en_name: "Burundi"},
    bj: %{code: 229, zh_name: "贝宁", en_name: "Benin"},
    bm: %{code: 1441, zh_name: "百慕大群岛", en_name: "Bermuda"},
    bn: %{code: 673, zh_name: "文莱", en_name: "Brunei"},
    bo: %{code: 591, zh_name: "玻利维亚", en_name: "Bolivia"},
    bq: %{code: 599, zh_name: "荷兰加勒比", en_name: "Caribisch Nederland"},
    br: %{code: 55, zh_name: "巴西", en_name: "Brazil"},
    bs: %{code: 1242, zh_name: "巴哈马", en_name: "Bahamas"},
    bt: %{code: 975, zh_name: "不丹", en_name: "Bhutan"},
    bw: %{code: 267, zh_name: "博茨瓦纳", en_name: "Botswana"},
    by: %{code: 375, zh_name: "白俄罗斯", en_name: "Belarus"},
    bz: %{code: 501, zh_name: "伯利兹", en_name: "Belize"},
    ca: %{code: 1, zh_name: "加拿大", en_name: "Canada"},
    cd: %{code: 243, zh_name: "刚果民主共和国", en_name: "Democratic Republic of theCongo"},
    cf: %{code: 236, zh_name: "中非共和国", en_name: "Central African Republic"},
    cg: %{code: 242, zh_name: "刚果共和国", en_name: "Republic Of The Congo"},
    ch: %{code: 41, zh_name: "瑞士", en_name: "Switzerland"},
    ci: %{code: 225, zh_name: "象牙海岸", en_name: "Ivory Coast"},
    ck: %{code: 682, zh_name: "库克群岛", en_name: "Cook Islands"},
    cl: %{code: 56, zh_name: "智利", en_name: "Chile"},
    cm: %{code: 237, zh_name: "喀麦隆", en_name: "Cameroon"},
    co: %{code: 57, zh_name: "哥伦比亚", en_name: "Colombia"},
    cr: %{code: 506, zh_name: "哥斯达黎加", en_name: "CostaRica"},
    cv: %{code: 238, zh_name: "开普", en_name: "Cape Verde"},
    cw: %{code: 599, zh_name: "库拉索", en_name: "Curacao"},
    cy: %{code: 357, zh_name: "塞浦路斯", en_name: "Cyprus"},
    cz: %{code: 420, zh_name: "捷克", en_name: "Czech"},
    de: %{code: 49, zh_name: "德国", en_name: "Germany"},
    dj: %{code: 253, zh_name: "吉布提", en_name: "Djibouti"},
    dk: %{code: 45, zh_name: "丹麦", en_name: "Denmark"},
    dm: %{code: 1767, zh_name: "多米尼加", en_name: "Dominica"},
    do: %{code: 1809, zh_name: "多米尼加共和国", en_name: "dominican republic"},
    dz: %{code: 213, zh_name: "阿尔及利亚", en_name: "Algeria"},
    ec: %{code: 593, zh_name: "厄瓜多尔", en_name: "Ecuador"},
    ee: %{code: 372, zh_name: "爱沙尼亚", en_name: "Estonia"},
    eg: %{code: 20, zh_name: "埃及", en_name: "Egypt"},
    er: %{code: 291, zh_name: "厄立特里亚", en_name: "Eritrea"},
    es: %{code: 34, zh_name: "西班牙", en_name: "Spain"},
    et: %{code: 251, zh_name: "埃塞俄比亚", en_name: "Ethiopia"},
    fi: %{code: 358, zh_name: "芬兰", en_name: "Finland"},
    fj: %{code: 679, zh_name: "斐济", en_name: "Fiji"},
    fm: %{code: 691, zh_name: "密克罗尼西亚", en_name: "Micronesia"},
    fo: %{code: 298, zh_name: "法罗群岛", en_name: "Faroe Islands"},
    fr: %{code: 33, zh_name: "法国", en_name: "France"},
    ga: %{code: 241, zh_name: "加蓬", en_name: "Gabon"},
    gb: %{code: 44, zh_name: "英国", en_name: "United Kingdom"},
    gd: %{code: 1473, zh_name: "格林纳达", en_name: "Grenada"},
    ge: %{code: 995, zh_name: "格鲁吉亚", en_name: "Georgia"},
    gf: %{code: 594, zh_name: "法属圭亚那", en_name: "French Guiana"},
    gh: %{code: 233, zh_name: "加纳", en_name: "Ghana"},
    gi: %{code: 350, zh_name: "直布罗陀", en_name: "Gibraltar"},
    gl: %{code: 299, zh_name: "格陵兰岛", en_name: "Greenland"},
    gm: %{code: 220, zh_name: "冈比亚", en_name: "Gambia"},
    gn: %{code: 224, zh_name: "几内亚", en_name: "Guinea"},
    gp: %{code: 590, zh_name: "瓜德罗普岛", en_name: "Guadeloupe"},
    gq: %{code: 240, zh_name: "赤道几内亚", en_name: "Equatorial Guinea"},
    gr: %{code: 30, zh_name: "希腊", en_name: "Greece"},
    gt: %{code: 502, zh_name: "瓜地马拉", en_name: "Guatemala"},
    gu: %{code: 1671, zh_name: "关岛", en_name: "Guam"},
    gw: %{code: 245, zh_name: "几内亚比绍共和国", en_name: "Guinea-Bissau"},
    gy: %{code: 592, zh_name: "圭亚那", en_name: "Guyana"},
    hk: %{code: 852, zh_name: "中国香港", en_name: "Hong Kong"},
    hn: %{code: 504, zh_name: "洪都拉斯", en_name: "Honduras"},
    hr: %{code: 385, zh_name: "克罗地亚", en_name: "Croatia"},
    ht: %{code: 509, zh_name: "海地", en_name: "Haiti"},
    hu: %{code: 36, zh_name: "匈牙利", en_name: "Hungary"},
    id: %{code: 62, zh_name: "印度尼西亚", en_name: "Indonesia"},
    ie: %{code: 353, zh_name: "爱尔兰", en_name: "Ireland"},
    il: %{code: 972, zh_name: "以色列", en_name: "Israel"},
    in: %{code: 91, zh_name: "印度", en_name: "India"},
    iq: %{code: 964, zh_name: "伊拉克", en_name: "Iraq"},
    is: %{code: 354, zh_name: "冰岛", en_name: "Iceland"},
    it: %{code: 39, zh_name: "意大利", en_name: "Italy"},
    jm: %{code: 1876, zh_name: "牙买加", en_name: "Jamaica"},
    jo: %{code: 962, zh_name: "约旦", en_name: "Jordan"},
    jp: %{code: 81, zh_name: "日本", en_name: "Japan"},
    ke: %{code: 254, zh_name: "肯尼亚", en_name: "Kenya"},
    kg: %{code: 996, zh_name: "吉尔吉斯斯坦", en_name: "Kyrgyzstan"},
    kh: %{code: 855, zh_name: "柬埔寨", en_name: "Cambodia"},
    ki: %{code: 686, zh_name: "基里巴斯", en_name: "Kiribati"},
    km: %{code: 269, zh_name: "科摩罗", en_name: "Comoros"},
    kn: %{code: 1869, zh_name: "圣基茨和尼维斯", en_name: "Saint Kitts and Nevis"},
    kr: %{code: 82, zh_name: "韩国", en_name: "South Korea"},
    kw: %{code: 965, zh_name: "科威特", en_name: "Kuwait"},
    ky: %{code: 1345, zh_name: "开曼群岛", en_name: "Cayman Islands"},
    kz: %{code: 7, zh_name: "哈萨克斯坦", en_name: "Kazakhstan"},
    la: %{code: 856, zh_name: "老挝", en_name: "Laos"},
    lb: %{code: 961, zh_name: "黎巴嫩", en_name: "Lebanon"},
    lc: %{code: 1758, zh_name: "圣露西亚", en_name: "Saint Lucia"},
    li: %{code: 423, zh_name: "列支敦士登", en_name: "Liechtenstein"},
    lk: %{code: 94, zh_name: "斯里兰卡", en_name: "Sri Lanka"},
    lr: %{code: 231, zh_name: "利比里亚", en_name: "Liberia"},
    ls: %{code: 266, zh_name: "莱索托", en_name: "Lesotho"},
    lt: %{code: 370, zh_name: "立陶宛", en_name: "Lithuania"},
    lu: %{code: 352, zh_name: "卢森堡", en_name: "Luxembourg"},
    lv: %{code: 371, zh_name: "拉脱维亚", en_name: "Latvia"},
    ly: %{code: 218, zh_name: "利比亚", en_name: "Libya"},
    ma: %{code: 212, zh_name: "摩洛哥", en_name: "Morocco"},
    mc: %{code: 377, zh_name: "摩纳哥", en_name: "Monaco"},
    md: %{code: 373, zh_name: "摩尔多瓦", en_name: "Moldova"},
    me: %{code: 382, zh_name: "黑山", en_name: "Montenegro"},
    mg: %{code: 261, zh_name: "马达加斯加", en_name: "Madagascar"},
    mh: %{code: 692, zh_name: "马绍尔群岛", en_name: "Marshall Islands"},
    mk: %{code: 389, zh_name: "马其顿", en_name: "Macedonia"},
    ml: %{code: 223, zh_name: "马里", en_name: "Mali"},
    mm: %{code: 95, zh_name: "缅甸", en_name: "Myanmar"},
    mn: %{code: 976, zh_name: "蒙古", en_name: "Mongolia"},
    mo: %{code: 853, zh_name: "中国澳门", en_name: "Macau"},
    mr: %{code: 222, zh_name: "毛里塔尼亚", en_name: "Mauritania"},
    ms: %{code: 1664, zh_name: "蒙特塞拉特岛", en_name: "Montserrat"},
    mt: %{code: 356, zh_name: "马耳他", en_name: "Malta"},
    mu: %{code: 230, zh_name: "毛里求斯", en_name: "Mauritius"},
    mv: %{code: 960, zh_name: "马尔代夫", en_name: "Maldives"},
    mw: %{code: 265, zh_name: "马拉维", en_name: "Malawi"},
    mx: %{code: 52, zh_name: "墨西哥", en_name: "Mexico"},
    my: %{code: 60, zh_name: "马来西亚", en_name: "Malaysia"},
    mz: %{code: 258, zh_name: "莫桑比克", en_name: "Mozambique"},
    na: %{code: 264, zh_name: "纳米比亚", en_name: "Namibia"},
    nc: %{code: 687, zh_name: "新喀里多尼亚", en_name: "New Caledonia"},
    ne: %{code: 227, zh_name: "尼日尔", en_name: "Niger"},
    ng: %{code: 234, zh_name: "尼日利亚", en_name: "Nigeria"},
    ni: %{code: 505, zh_name: "尼加拉瓜", en_name: "Nicaragua"},
    nl: %{code: 31, zh_name: "荷兰", en_name: "Netherlands"},
    no: %{code: 47, zh_name: "挪威", en_name: "Norway"},
    np: %{code: 977, zh_name: "尼泊尔", en_name: "Nepal"},
    nr: %{code: 674, zh_name: "拿鲁岛", en_name: "Nauru"},
    nz: %{code: 64, zh_name: "新西兰", en_name: "New Zealand"},
    om: %{code: 968, zh_name: "阿曼", en_name: "Oman"},
    pa: %{code: 507, zh_name: "巴拿马", en_name: "Panama"},
    pe: %{code: 51, zh_name: "秘鲁", en_name: "Peru"},
    pf: %{code: 689, zh_name: "法属波利尼西亚", en_name: "French Polynesia"},
    pg: %{code: 675, zh_name: "巴布亚新几内亚", en_name: "Papua New Guinea"},
    ph: %{code: 63, zh_name: "菲律宾", en_name: "Philippines"},
    pk: %{code: 92, zh_name: "巴基斯坦", en_name: "Pakistan"},
    pl: %{code: 48, zh_name: "波兰", en_name: "Poland"},
    pm: %{code: 508, zh_name: "圣彼埃尔和密克隆岛", en_name: "Saint Pierreand Miquelon"},
    pr: %{code: 1787, zh_name: "波多黎各", en_name: "Puerto Rico"},
    pt: %{code: 351, zh_name: "葡萄牙", en_name: "Portugal"},
    pw: %{code: 680, zh_name: "帕劳", en_name: "Palau"},
    py: %{code: 595, zh_name: "巴拉圭", en_name: "Paraguay"},
    qa: %{code: 974, zh_name: "卡塔尔", en_name: "Qatar"},
    re: %{code: 262, zh_name: "留尼汪", en_name: "Réunion Island"},
    ro: %{code: 40, zh_name: "罗马尼亚", en_name: "Romania"},
    rs: %{code: 381, zh_name: "塞尔维亚", en_name: "Serbia"},
    ru: %{code: 7, zh_name: "俄罗斯", en_name: "Russia"},
    rw: %{code: 250, zh_name: "卢旺达", en_name: "Rwanda"},
    sa: %{code: 966, zh_name: "沙特阿拉伯", en_name: "Saudi Arabia"},
    sb: %{code: 677, zh_name: "所罗门群岛", en_name: "Solomon Islands"},
    sc: %{code: 248, zh_name: "塞舌尔", en_name: "Seychelles"},
    sd: %{code: 249, zh_name: "苏丹", en_name: "Sudan"},
    se: %{code: 46, zh_name: "瑞典", en_name: "Sweden"},
    sg: %{code: 65, zh_name: "新加坡", en_name: "Singapore"},
    si: %{code: 386, zh_name: "斯洛文尼亚", en_name: "Slovenia"},
    sk: %{code: 421, zh_name: "斯洛伐克", en_name: "Slovakia"},
    sl: %{code: 232, zh_name: "塞拉利昂", en_name: "Sierra Leone"},
    sm: %{code: 378, zh_name: "圣马力诺", en_name: "San Marino"},
    sn: %{code: 221, zh_name: "塞内加尔", en_name: "Senegal"},
    so: %{code: 252, zh_name: "索马里", en_name: "Somalia"},
    sr: %{code: 597, zh_name: "苏里南", en_name: "Suriname"},
    st: %{code: 239, zh_name: "圣多美和普林西比", en_name: "Sao Tome and Principe"},
    sv: %{code: 503, zh_name: "萨尔瓦多", en_name: "ElSalvador"},
    sz: %{code: 268, zh_name: "斯威士兰", en_name: "Swaziland"},
    tc: %{code: 1649, zh_name: "特克斯和凯科斯群岛", en_name: "Turksand Caicos Islands"},
    td: %{code: 235, zh_name: "乍得", en_name: "Chad"},
    tg: %{code: 228, zh_name: "多哥", en_name: "Togo"},
    th: %{code: 66, zh_name: "泰国", en_name: "Thailand"},
    tj: %{code: 992, zh_name: "塔吉克斯坦", en_name: "Tajikistan"},
    tl: %{code: 670, zh_name: "东帝汶", en_name: "East Timor"},
    tm: %{code: 993, zh_name: "土库曼斯坦", en_name: "Turkmenistan"},
    tn: %{code: 216, zh_name: "突尼斯", en_name: "Tunisia"},
    to: %{code: 676, zh_name: "汤加", en_name: "Tonga"},
    tr: %{code: 90, zh_name: "土耳其", en_name: "Turkey"},
    tt: %{code: 1868, zh_name: "特立尼达和多巴哥", en_name: "Trinidadand Tobago"},
    tw: %{code: 886, zh_name: "中国台湾", en_name: "Taiwan"},
    tz: %{code: 255, zh_name: "坦桑尼亚", en_name: "Tanzania"},
    ua: %{code: 380, zh_name: "乌克兰", en_name: "Ukraine"},
    ug: %{code: 256, zh_name: "乌干达", en_name: "Uganda"},
    us: %{code: 1, zh_name: "美国", en_name: "United States"},
    uy: %{code: 598, zh_name: "乌拉圭", en_name: "Uruguay"},
    uz: %{code: 998, zh_name: "乌兹别克斯坦", en_name: "Uzbekistan"},
    vc: %{code: 1784, zh_name: "圣文森特和格林纳丁斯", en_name: "Saint Vincent and The Grenadines"},
    ve: %{code: 58, zh_name: "委内瑞拉", en_name: "Venezuela"},
    vg: %{code: 1284, zh_name: "英属处女群岛", en_name: "VirginIslands,British"},
    vn: %{code: 84, zh_name: "越南", en_name: "Vietnam"},
    vu: %{code: 678, zh_name: "瓦努阿图", en_name: "Vanuatu"},
    ws: %{code: 685, zh_name: "萨摩亚", en_name: "Samoa"},
    ye: %{code: 967, zh_name: "也门", en_name: "Yemen"},
    yt: %{code: 269, zh_name: "马约特", en_name: "Mayotte"},
    za: %{code: 27, zh_name: "南非", en_name: "South Africa"},
    zm: %{code: 260, zh_name: "赞比亚", en_name: "Zambia"},
    zw: %{code: 263, zh_name: "津巴布韦", en_name: "Zimbabwe"}
  ]

  def nations, do: @nations
end
